{-----------------------------------------------------------------------------
   Unit Name: UnitComboBox
   Purpose  : ComboBox that can help doing transforms between units
   Author   : Vesa Lappalainen
   Date     : 17.12.1997
   Changed  : 04.11.2001
    + CLX compiling
    + default array property to TUnits
   Changed  : 13.11.2001
    + DecimalSeparator!
   Todo     :
    + Combos should join to TUnits. Then in case of deleting TUnits
      there would be no problems.  There is also need to get information
      when Units changes

   Komponentti, jossa jokaista merkkijonoa vastaa kerroin (factor)
   ja siirtym (Shift).  Voidaan kytt erilaisissa muunnoksissa.

   Aluksi on toteutettu apuluokka TOneUnit, jossa on yhden
   muunnoksen vaatimat tiedot.

   Seuraava luokka TUnits on nkymtn komponentti, jonka Items
   ominaisuuteen voidaan kirjoittaa muunnokset muodossa:
   mm,1          tai  K,0,0
   cm,10              C,1,273

   Muunnokset tehdn aina suhteessa johonkin perusyksikkn.

   Jokaisessa TUnitComboBox -oliossa voi olla sitten viite yhteen
   TUnits -olioon.

-----------------------------------------------------------------------------}

unit UnitComboBox;

interface

uses
  SysUtils,
{$ifdef CLX}
  QGraphics, QControls, QForms, QDialogs,  QStdCtrls,
{$else}
  Windows, Messages,
  Graphics, Controls, Forms, Dialogs,  StdCtrls,
{$endif}
  Classes;

type
  //--------------------------------------------------------------
  TOneUnit = class
    FName   : string;
    FFactor : double;
    FShift  : double;
  private
  public
    constructor Create(const s:string); virtual;
  published
    property Name   : string read FName;
    property Factor : double read FFactor;
    property Shift  : double read FShift;
  end;

  //--------------------------------------------------------------
  TUnits = class(TComponent)
  private
    FItems : TStringList;
    FUnits : TStrings;
    procedure ItemsChange(Sender: TObject);
  public
    constructor Create(AOwner:TComponent); override;
    destructor Destroy; override;
    procedure Recalc; virtual;
    procedure Clear; virtual;
    procedure SetItems(s:TStringList); virtual;
    function GetOneUnit(i:integer):TOneUnit; virtual;
    property OneUnit[i:integer]:TOneUnit read GetOneUnit; default;
    property Units : TStrings read FUnits;
  published
    property Items : TStringList read FItems write SetItems;
  end;

  //--------------------------------------------------------------
  TUnitComboBox = class(TComboBox)
  private
    FUnits : TUnits;
  protected
  public
    procedure Recalc; virtual;
    procedure SetUnits(u : TUnits); virtual;
    function GetFactor : double;
    function GetShift : double;
    function GetUnit : TOneUnit;
    property Factor : double read GetFactor;
    property Shift : double read GetShift;
    property OneUnit : TOneUnit read GetUnit;
    function Code(d:double) : double;
    function UnCode(d:double) : double;
  published
    property Units : TUnits read FUnits write SetUnits default nil;
  end;

procedure Register;

implementation
uses kdouble;

//------------------------------------------------------------------------------
// TOneUnit
//------------------------------------------------------------------------------
constructor TOneUnit.Create(const s:string);
var st : TStrings;
begin
  st := TStringList.Create;
  st.CommaText := s+',,,';
  FName   := st[0];
  FFactor := StrToDoubleDef(st[1],1);
  FShift  := StrToDoubleDef(st[2],0);
  st.Free;
end;

//------------------------------------------------------------------------------
// TUnits
//------------------------------------------------------------------------------
constructor TUnits.Create(AOwner:TComponent);
begin
  inherited;
  FItems := TStringList.Create;
  FUnits := TStringList.Create;
  FItems.OnChange := ItemsChange;
end;

destructor TUnits.Destroy;
begin
  Clear;
  FItems.Free;
  FUnits.Free;
  inherited;
end;

procedure TUnits.Clear;
var i:integer;
begin
  for i:=0 to Units.Count-1 do
    if ( Units.Objects[i] <> nil ) then begin
      Units.Objects[i].Free;
      Units.Objects[i] := nil;
    end;
  Units.Clear;
end;

procedure TUnits.Recalc;
var i:integer; u : TOneUnit; oldsep : char;
begin
  Clear;
  oldsep := DecimalSeparator;
  DecimalSeparator := '.';
  for i:=0 to Items.Count-1 do begin
    u := TOneUnit.Create(Items[i]);
    Units.AddObject(u.Name,u);
  end;
  DecimalSeparator := oldsep;
end;

procedure TUnits.SetItems(s:TStringList);
begin
  FItems.Assign(s);
  Recalc;
end;

procedure TUnits.ItemsChange(Sender: TObject);
begin
  Recalc;
end;

function TUnits.GetOneUnit(i: integer): TOneUnit;
begin
  Result := nil;
  if ( i < 0 ) then exit;
  if ( i >= Units.Count ) then exit;
  if not ( Units.Objects[i] is TOneUnit ) then exit;
  Result := TOneUnit(Units.Objects[i]);
end;


//------------------------------------------------------------------------------
// TUnitComboBox
//------------------------------------------------------------------------------
procedure TUnitComboBox.Recalc;
begin
  Clear;
  Items := Units.Units;
end;

procedure TUnitComboBox.SetUnits(u : TUnits);
var oldText:string;
begin
  oldText := Text;
  FUnits := u;
  Recalc;
//  Text := oldText;
  ItemIndex := Items.IndexOf(oldText);
end;

function TUnitComboBox.GetUnit : TOneUnit;
var i:integer;
begin
  Result := nil;
  if ( not Assigned(Units) ) then exit;
  i := ItemIndex;
  Result := Units[i];
end;

function TUnitComboBox.GetFactor : double;
var u : TOneUnit;
begin
  Result := 0;
  u := OneUnit; if ( u = nil ) then exit;
  Result := u.Factor;
end;

function TUnitComboBox.GetShift : double;
var u : TOneUnit;
begin
  Result := 0;
  u := OneUnit; if ( u = nil ) then exit;
  Result := u.Shift;
end;

function TUnitComboBox.Code(d:double) : double;
begin
  Result := d * Factor + Shift;
end;

function TUnitComboBox.UnCode(d:double) : double;
begin
  Result := 0;
  if ( Factor = 0 ) then exit;
  Result := (d - Shift)/Factor;
end;

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
procedure Register;
begin
  RegisterComponents('GKO', [TUnits,TUnitComboBox]);
end;

end.
